#!/bin/bash

PKG_LIST=(
    sysbench_1.0.20+ds-1_amd64.deb
    libpq5_13.5-0+deb11u1_amd64.deb
    libluajit-5.1-2_2.1.0~beta3+dfsg-5.3_amd64.deb
    sysbench.tar.gz
)

log() {
    local text;local logtype
    logtype=$1
    text=$2
    case $logtype in 
        error)
            echo -e "\033[31m`date +'%F %H:%M:%S'`[$1]\t$2\033[0m" | tee -a $recovery_backup_log;;
        info)
            echo -e "\033[36m`date +'%F %H:%M:%S'`[$1]\t$2\033[0m" | tee -a $recovery_backup_log;;
        warn)
            echo -e "\033[33m`date +'%F %H:%M:%S'`[$1]\t$2\033[0m" | tee -a $recovery_backup_log;;
    esac
}

exec_on_container() {
    container_id=$(docker ps |grep mysql:v8 |awk '{print $1}')
    exec_sql="mysql -urepl -pXjb0^DDy -hmysql -N -e"

    if [ "$1" == "sql" ];then
        docker exec  $container_id $exec_sql "$2"
    else
        docker exec  $container_id $*
    fi
}

check_mgr() {
    database_type=$(exec_on_container sql "SELECT CASE WHEN COUNT(PLUGIN_STATUS)=0 THEN 'standalone' \
    WHEN PLUGIN_STATUS='ACTIVE' THEN 'cluster'  END AS deploy_mode \
    FROM  INFORMATION_SCHEMA.PLUGINS WHERE PLUGIN_NAME = 'group_replication';")

    if [ "$database_type" == "cluster" ];then
        res=$(exec_on_container sql "select * from performance_schema.replication_group_members")
        health_num=$(echo $res | grep -o ONLINE |wc -l)
        if [ "$health_num" != "3" ];then
            log error "MGR 集群异常，当前节点数量为: $health_num.请修复节点状态后再执行压测脚本."
            exit 1 ;
        else
            log info "MGR 集群节点状态正常，当前节点数量为: $health_num."
        fi
    fi
}


install_tools() {
    for i in ${PKG_LIST[@]};do
        ls ./ |grep $i  > /dev/null 2>&1
        if [ $? -eq 0 ];then
            log info "$i install pakg exist."
            docker cp $i $(docker ps |grep mysql: |awk {'print $1'}):/tmp
            exec_on_container dpkg -i /tmp/$i
        else
            log error "$i install pakg doesn't exist."
            exit 1;
        fi
    done


    exec_on_container tar -zxf /tmp/sysbench.tar.gz -C /tmp/
}

create_data() {
    check_mgr
    exec_on_container sql "DROP DATABASE IF EXISTS sbtest ; CREATE DATABASE sbtest;"
    exec_on_container sysbench /usr/share/sysbench/oltp_common.lua --mysql-host=mysql --mysql-port=3306 --mysql-user=repl --mysql-password=Xjb0^DDy --tables=${tables:-10} --table-size=${tables_size:-1000}  --threads=${threads:-500} --time=${time:-600} --report-interval=0 ${action}
    [ $? -eq 0 ] && log info "创建完成.使用 -a/--action run 参数进行测试."
}

start_bench() {
    check_mgr
    log info "压测表个数: ${tables:-10}"
    log info "压测表数据量: ${tables_size:-1000}"
    log info "压测并发数: ${threads:-500}"
    log info "压测时间: ${time:-600}"

    bench_res=$(exec_on_container sysbench /tmp/sysbench/tests/include/oltp_legacy/oltp.lua --mysql-host=mysql --mysql-port=3306 --mysql-user=repl --mysql-password=Xjb0^DDy --tables=${tables:-10} --table-size=${tables_size:-10000000}  --threads=${threads:-500} --time=${time:-600} --report-interval=0 ${action})

    qps=$(echo $bench_res|grep -oE 'queries:\s[0-9a-zA-Z\_]+\s[(][0-9a-zA-Z\_]+' | awk -F"(" '{print $2}')
    tps=$(echo $bench_res|grep -oE 'transactions:\s[0-9a-zA-Z\_]+\s[(][0-9a-zA-Z\_]+' | awk -F"(" '{print $2}')
    query_time=$(echo $bench_res|grep -oE 'percentile:\s[0-9a-zA-Z\_]+' | awk '{print $2}')
    
    log info "###测试结果:###"
    log info "QPS: $qps"
    log info "TPS: $tps"
    log info "百分之95事务耗时: $query_time ms"

    if [ $qps -lt 40000 ];then
        log warn "QPS 小于 40000. 数据库读写性能不足."
    else
        log info "QPS 大于 40000.为正常范围."
    fi

    if [ $query_time -gt 400 ]; then
        log warn "百分之95事务耗时大于 400 ms.数据库读写性能不足."
    else
        log info "百分之95事务耗时小于 400 ms.为正常范围."
    fi

}

cleanup_data() {
    check_mgr
    exec_on_container sql "DROP DATABASE IF EXISTS sbtest ;"
}

help_str="
使用方式:
    如无特殊需求，无需设定 -a/--action 以外参数
    sh sysbench.sh -a [ prepare | run | cleanup ]
参数说明:
  -h, --help:           打印帮助信息
  -o, --tables:         设置测试表个数，默认10个
  -s, --tables-size:    设置表行数，默认1000万
  -c, --threads:        设置并发线程数，默认500
  -t, --time:           设置压测时间
  -a, --action:         设置执行操作。prepare 生成压测数据; run 进行压力测试; cleanup 删除压测数据
"
declare -a names 

# ARGS=`getopt -o o::s::c::t::a: --long tables::,tables-size::,threads::,time::,action: "$@"`
# getopt_cmd=$(getopt -o o:s:c:t:a: --long tables:,tables-size:,threads:,time:,action: -n $(basename $0) -- "$@")
getopt_cmd=$(getopt -o a:o:s:c:t:h --long action:,tables:,tables-size:,threads:,time:,help -n $(basename $0) -- "$@")

[ $? -ne 0 ] && exit 1

eval set -- "$getopt_cmd"
# 解析选项
while [ -n "$1" ]
do
    case "$1" in
        -a|--action)
            action="$2"
            shift ;;
        -o|--tables)
            tables="$2"
            shift ;;
        -s|--tables-size)
            tables_size="$2"
            shift ;;
        -c|--threads)
            threads="$2"
            shift ;;
        -t|--time)
            time="$2"
            shift ;;
        -h|--help)
            echo -e "$help_str"
            exit ;;
        --)
            shift
            break
            ;;
        *)
            echo "error"
            # log error "Parameter error!"
            exit 1 ;;
    esac
    shift
done

# 解析参数
while [ -n "$1" ]
do
    names=("${names[@]}" "$1")
    shift
done


# echo ${tables:-10}
# echo ${tables_size:-10000000}
# echo ${time}
# echo ${action}

sysbench_ver=$(exec_on_container sysbench --version)
if [ $? -eq 0 ]; then
    log info "sysbench 压测工具已经安装,版本为: $sysbench_ver"
else
    log info "安装 sysbench 压测工具..."
    install_tools
fi

case $action in
    prepare)
        log info "现在开始创建测试数据."
        create_data
        ;;
    run)
        log info "现在开始进行压力测试."
        start_bench
        ;;
    cleanup)
        cleanup_data
        log info "删除测试数据."
        ;;
    *)
        log error "请正确设置 -a/--action 参数,指定脚本执行操作,可选[ prepare | run | cleanup ] "
        ;;
esac

